@@ -36,6 +36,7 @@ from typing import (
 from urllib import parse
 
 from playwright._impl._api_structures import (
+    ClientCertificate,
     Headers,
     HeadersArray,
     RemoteAddr,
@@ -50,7 +51,7 @@ from playwright._impl._connection import (
 )
 from playwright._impl._errors import Error
 from playwright._impl._event_context_manager import EventContextManagerImpl
-from playwright._impl._helper import locals_to_params
+from playwright._impl._helper import async_readfile, locals_to_params
 from playwright._impl._waiter import Waiter
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -83,6 +84,34 @@ def serialize_headers(headers: Dict[str, str]) -> HeadersArray:
     ]
 
 
+async def to_client_certificates_protocol(
+    clientCertificates: Optional[List[ClientCertificate]],
+) -> Optional[List[Dict[str, str]]]:
+    if not clientCertificates:
+        return None
+    out = []
+    for clientCertificate in clientCertificates:
+        out_record = {
+            "origin": clientCertificate["origin"],
+        }
+        if passphrase := clientCertificate.get("passphrase"):
+            out_record["passphrase"] = passphrase
+        if pfx_path := clientCertificate.get("pfxPath"):
+            out_record["pfx"] = base64.b64encode(
+                await async_readfile(pfx_path)
+            ).decode()
+        if cert_path := clientCertificate.get("certPath"):
+            out_record["cert"] = base64.b64encode(
+                await async_readfile(cert_path)
+            ).decode()
+        if key_path := clientCertificate.get("keyPath"):
+            out_record["key"] = base64.b64encode(
+                await async_readfile(key_path)
+            ).decode()
+        out.append(out_record)
+    return out
+
+
 class Request(ChannelOwner):
     def __init__(
         self, parent: ChannelOwner, type: str, guid: str, initializer: Dict
@@ -410,6 +439,7 @@ class Route(ChannelOwner):
         headers: Dict[str, str] = None,
         postData: Union[Any, str, bytes] = None,
         maxRedirects: int = None,
+        maxRetries: int = None,
         timeout: float = None,
     ) -> "APIResponse":
         return await self._connection.wrap_api_call(
@@ -420,6 +450,7 @@ class Route(ChannelOwner):
                 headers,
                 postData,
                 maxRedirects=maxRedirects,
+                maxRetries=maxRetries,
                 timeout=timeout,
             )
         )
